@@ -17,18 +17,19 @@ from novaclient.tests.unit.v2 import fakes
 from novaclient.v2.contrib import list_extensions
 
 
-extensions = [
-    extension.Extension(list_extensions.__name__.split(".")[-1],
-                        list_extensions),
-]
-cs = fakes.FakeClient(extensions=extensions)
-
-
 class ListExtensionsTests(utils.TestCase):
+    def setUp(self):
+        super(ListExtensionsTests, self).setUp()
+        extensions = [
+            extension.Extension(list_extensions.__name__.split(".")[-1],
+                                list_extensions),
+        ]
+        self.cs = fakes.FakeClient(extensions=extensions)
+
     def test_list_extensions(self):
-        all_exts = cs.list_extensions.show_all()
+        all_exts = self.cs.list_extensions.show_all()
         self.assert_request_id(all_exts, fakes.FAKE_REQUEST_ID_LIST)
-        cs.assert_called('GET', '/extensions')
+        self.cs.assert_called('GET', '/extensions')
         self.assertTrue(len(all_exts) > 0)
         for r in all_exts:
             self.assertTrue(len(r.summary) > 0)
