@@ -50,18 +50,25 @@ class Server(base.Resource):
         """
         return self.manager.delete(self)
 
+    @api_versions.wraps("2.0", "2.18")
+    def update(self, name=None):
+        """
+        Update the name and the description for this server.
+
+        :param name: Update the server's name.
+        :returns: :class:`Server`
+        """
+        return self.manager.update(self, name=name)
+
+    @api_versions.wraps("2.19")
     def update(self, name=None, description=None):
         """
         Update the name and the description for this server.
 
         :param name: Update the server's name.
-        :param description: Update the server's description(
-                            allowed for 2.19-latest).
+        :param description: Update the server's description.
         :returns: :class:`Server`
         """
-        if (description is not None and
-                self.manager.api_version < api_versions.APIVersion("2.19")):
-            raise exceptions.UnsupportedAttribute("description", "2.19")
         update_kwargs = {"name": name}
         if description is not None:
             update_kwargs["description"] = description
@@ -403,43 +410,56 @@ class Server(base.Resource):
         except Exception:
             return {}
 
+    @api_versions.wraps("2.0", "2.24")
     def live_migrate(self, host=None,
-                     block_migration=None,
+                     block_migration=False,
                      disk_over_commit=None):
         """
         Migrates a running instance to a new machine.
 
         :param host: destination host name.
         :param block_migration: if True, do block_migration, the default
-                                value None will be mapped to False for 2.0 -
-                                2.24, 'auto' for higher than 2.25
+                                value is False and None will be mapped to False
         :param disk_over_commit: if True, allow disk over commit, the default
-                                 value None will be mapped to False. It will
-                                 not be supported since 2.25
+                                 value is None which is mapped to False
         :returns: An instance of novaclient.base.TupleWithMeta
         """
+        if block_migration is None:
+            block_migration = False
+        if disk_over_commit is None:
+            disk_over_commit = False
+        return self.manager.live_migrate(self, host,
+                                         block_migration,
+                                         disk_over_commit)
 
-        if (self.manager.api_version < api_versions.APIVersion("2.25")):
-            # NOTE(eliqiao): We do this to keep old version api has same
-            # default value if user don't pass these parameters when using
-            # SDK
-            if block_migration is None:
-                block_migration = False
-            if disk_over_commit is None:
-                disk_over_commit = False
+    @api_versions.wraps("2.25", "2.29")
+    def live_migrate(self, host=None, block_migration=None):
+        """
+        Migrates a running instance to a new machine.
 
-            return self.manager.live_migrate(self, host,
-                                             block_migration,
-                                             disk_over_commit)
-        else:
-            if block_migration is None:
-                block_migration = 'auto'
-            if disk_over_commit is not None:
-                raise ValueError("Setting 'disk_over_commit' argument is "
-                                 "prohibited after microversion 2.25.")
+        :param host: destination host name.
+        :param block_migration: if True, do block_migration, the default
+                                value is None which is mapped to 'auto'.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+        if block_migration is None:
+            block_migration = "auto"
+        return self.manager.live_migrate(self, host, block_migration)
 
-            return self.manager.live_migrate(self, host,
-                                             block_migration)
+    @api_versions.wraps("2.30")
+    def live_migrate(self, host=None, block_migration=None, force=None):
+        """
+        Migrates a running instance to a new machine.
+
+        :param host: destination host name.
+        :param block_migration: if True, do block_migration, the default
+                                value is None which is mapped to 'auto'.
+        :param force: force to bypass the scheduler if host is provided.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+        if block_migration is None:
+            block_migration = "auto"
+        return self.manager.live_migrate(self, host, block_migration, force)
 
     def reset_state(self, state='error'):
         """
@@ -480,29 +500,44 @@ class Server(base.Resource):
         """
         return self.manager.list_security_group(self)
 
-    def evacuate(self, host=None, on_shared_storage=None, password=None):
+    @api_versions.wraps("2.0", "2.13")
+    def evacuate(self, host=None, on_shared_storage=True, password=None):
         """
         Evacuate an instance from failed host to specified host.
 
         :param host: Name of the target host
         :param on_shared_storage: Specifies whether instance files located
-                        on shared storage. After microversion 2.14, this
-                        parameter must have its default value of None.
+                        on shared storage.
         :param password: string to set as admin password on the evacuated
                          server.
         :returns: An instance of novaclient.base.TupleWithMeta
         """
-        if api_versions.APIVersion("2.14") <= self.manager.api_version:
-            if on_shared_storage is not None:
-                raise ValueError("Setting 'on_shared_storage' argument is "
-                                 "prohibited after microversion 2.14.")
-            return self.manager.evacuate(self, host, password)
-        else:
-            # microversions 2.0 - 2.13
-            if on_shared_storage is None:
-                on_shared_storage = True
-            return self.manager.evacuate(self, host, on_shared_storage,
-                                         password)
+        return self.manager.evacuate(self, host, on_shared_storage, password)
+
+    @api_versions.wraps("2.14", "2.28")
+    def evacuate(self, host=None, password=None):
+        """
+        Evacuate an instance from failed host to specified host.
+
+        :param host: Name of the target host
+        :param password: string to set as admin password on the evacuated
+                         server.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+        return self.manager.evacuate(self, host, password)
+
+    @api_versions.wraps("2.29")
+    def evacuate(self, host=None, password=None, force=None):
+        """
+        Evacuate an instance from failed host to specified host.
+
+        :param host: Name of the target host
+        :param password: string to set as admin password on the evacuated
+                         server.
+        :param force: forces to bypass the scheduler if host is provided.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+        return self.manager.evacuate(self, host, password, force)
 
     def interface_list(self):
         """
@@ -526,6 +561,41 @@ class Server(base.Resource):
         """Trigger crash dump in an instance"""
         return self.manager.trigger_crash_dump(self)
 
+    @api_versions.wraps('2.26')
+    def tag_list(self):
+        """
+        Get list of tags from an instance.
+        """
+        return self.manager.tag_list(self)
+
+    @api_versions.wraps('2.26')
+    def delete_tag(self, tag):
+        """
+        Remove single tag from an instance.
+        """
+        return self.manager.delete_tag(self, tag)
+
+    @api_versions.wraps('2.26')
+    def delete_all_tags(self):
+        """
+        Remove all tags from an instance.
+        """
+        return self.manager.delete_all_tags(self)
+
+    @api_versions.wraps('2.26')
+    def set_tags(self, tags):
+        """
+        Set list of tags to an instance.
+        """
+        return self.manager.set_tags(self, tags)
+
+    @api_versions.wraps('2.26')
+    def add_tag(self, tag):
+        """
+        Add single tag to an instance.
+        """
+        return self.manager.add_tag(self, tag)
+
 
 class NetworkInterface(base.Resource):
     @property
@@ -657,6 +727,8 @@ class ServerManager(base.BootingManagerWithFind):
                     net_data['fixed_ip'] = nic_info['v6-fixed-ip']
                 if nic_info.get('port-id'):
                     net_data['port'] = nic_info['port-id']
+                if nic_info.get('tag'):
+                    net_data['tag'] = nic_info['tag']
                 all_net_data.append(net_data)
             body['server']['networks'] = all_net_data
 
@@ -1217,6 +1289,22 @@ class ServerManager(base.BootingManagerWithFind):
         if "description" in kwargs and self.api_version < descr_microversion:
             raise exceptions.UnsupportedAttribute("description", "2.19")
 
+        tags_microversion = api_versions.APIVersion("2.32")
+        if self.api_version < tags_microversion:
+            if nics:
+                for nic_info in nics:
+                    if nic_info.get("tag"):
+                        raise ValueError("Setting interface tags is "
+                                         "unsupported before microversion "
+                                         "2.32")
+
+            if block_device_mapping_v2:
+                for bdm in block_device_mapping_v2:
+                    if bdm.get("tag"):
+                        raise ValueError("Setting block device tags is "
+                                         "unsupported before microversion "
+                                         "2.32")
+
         boot_kwargs = dict(
             meta=meta, files=files, userdata=userdata,
             reservation_id=reservation_id, min_count=min_count,
@@ -1523,7 +1611,7 @@ class ServerManager(base.BootingManagerWithFind):
                              'block_migration': block_migration,
                              'disk_over_commit': disk_over_commit})
 
-    @api_versions.wraps('2.25')
+    @api_versions.wraps('2.25', '2.29')
     def live_migrate(self, server, host, block_migration):
         """
         Migrates a running instance to a new machine.
@@ -1538,6 +1626,23 @@ class ServerManager(base.BootingManagerWithFind):
                             {'host': host,
                              'block_migration': block_migration})
 
+    @api_versions.wraps('2.30')
+    def live_migrate(self, server, host, block_migration, force=None):
+        """
+        Migrates a running instance to a new machine.
+
+        :param server: instance id which comes from nova list.
+        :param host: destination host name.
+        :param block_migration: if True, do block_migration, can be set as
+                                'auto'
+        :param force: forces to bypass the scheduler if host is provided.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+        body = {'host': host, 'block_migration': block_migration}
+        if force:
+            body['force'] = force
+        return self._action('os-migrateLive', server, body)
+
     def reset_state(self, server, state='error'):
         """
         Reset the state of an instance to active or error.
@@ -1616,7 +1721,7 @@ class ServerManager(base.BootingManagerWithFind):
                                                        body)
         return base.TupleWithMeta((resp, body), resp)
 
-    @api_versions.wraps("2.14")
+    @api_versions.wraps("2.14", "2.28")
     def evacuate(self, server, host=None, password=None):
         """
         Evacuate a server instance.
@@ -1638,6 +1743,32 @@ class ServerManager(base.BootingManagerWithFind):
                                                        body)
         return base.TupleWithMeta((resp, body), resp)
 
+    @api_versions.wraps("2.29")
+    def evacuate(self, server, host=None, password=None, force=None):
+        """
+        Evacuate a server instance.
+
+        :param server: The :class:`Server` (or its ID) to share onto.
+        :param host: Name of the target host.
+        :param password: string to set as password on the evacuated server.
+        :param force: forces to bypass the scheduler if host is provided.
+        :returns: An instance of novaclient.base.TupleWithMeta
+        """
+
+        body = {}
+        if host is not None:
+            body['host'] = host
+
+        if password is not None:
+            body['adminPass'] = password
+
+        if force:
+            body['force'] = force
+
+        resp, body = self._action_return_resp_and_body('evacuate', server,
+                                                       body)
+        return base.TupleWithMeta((resp, body), resp)
+
     def interface_list(self, server):
         """
         List attached network interfaces
@@ -1709,3 +1840,42 @@ class ServerManager(base.BootingManagerWithFind):
         url = '/servers/%s/remote-consoles' % base.getid(server)
         resp, body = self.api.client.post(url, body=body)
         return self.convert_into_with_meta(body, resp)
+
+    @api_versions.wraps('2.26')
+    def tag_list(self, server):
+        """
+        Get list of tags from an instance.
+        """
+        resp, body = self.api.client.get(
+            "/servers/%s/tags" % base.getid(server))
+        return base.ListWithMeta(body['tags'], resp)
+
+    @api_versions.wraps('2.26')
+    def delete_tag(self, server, tag):
+        """
+        Remove single tag from an instance.
+        """
+        return self._delete("/servers/%s/tags/%s" % (base.getid(server), tag))
+
+    @api_versions.wraps('2.26')
+    def delete_all_tags(self, server):
+        """
+        Remove all tags from an instance.
+        """
+        return self._delete("/servers/%s/tags" % base.getid(server))
+
+    @api_versions.wraps('2.26')
+    def set_tags(self, server, tags):
+        """
+        Set list of tags to an instance.
+        """
+        body = {"tags": tags}
+        return self._update("/servers/%s/tags" % base.getid(server), body)
+
+    @api_versions.wraps('2.26')
+    def add_tag(self, server, tag):
+        """
+        Add single tag to an instance.
+        """
+        return self._update(
+            "/servers/%s/tags/%s" % (base.getid(server), tag), None)
